## $Id: browser.py 13562 2016-01-04 07:54:31Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import grok
from zope.component import getUtility
from waeup.kofa.students.interfaces import IStudentsUtils
from kofacustom.nigeria.interswitch.browser import (
    InterswitchPaymentRequestWebservicePageApplicant,
    InterswitchPaymentRequestWebservicePageStudent,
    InterswitchPageStudent, InterswitchPageApplicant,
    )
from waeup.uniben.students.interfaces import ICustomStudentOnlinePayment
from waeup.uniben.applicants.interfaces import ICustomApplicantOnlinePayment
from waeup.uniben.interfaces import MessageFactory as _

PRODUCT_ID = '57'
SITE_NAME = 'uniben-kofa.waeup.org'
PROVIDER_ACCT = '1010764827'
PROVIDER_BANK_ID = '117'
PROVIDER_ITEM_NAME = 'BT Education'
INSTITUTION_NAME = 'Uniben'
CURRENCY = '566'
GATEWAY_AMT = 150.0
#QUERY_URL = 'https://webpay.interswitchng.com/paydirect/services/TransactionQueryURL.aspx'
#QUERY_URL = 'https://testwebpay.interswitchng.com/test_paydirect/services/TransactionQueryURL.aspx'
POST_ACTION = 'https://webpay.interswitchng.com/paydirect/webpay/pay.aspx'
#POST_ACTION = 'https://testwebpay.interswitchng.com/test_paydirect/webpay/pay.aspx'

HOST = 'webpay.interswitchng.com'
#HOST = 'testwebpay.interswitchng.com'
URL = '/paydirect/services/TransactionQueryWs.asmx'
#URL = '/test_paydirect/services/TransactionQueryWs.asmx'
httplib.HTTPConnection.debuglevel = 0

class CustomInterswitchPageStudent(InterswitchPageStudent):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    product_id = PRODUCT_ID

    def update(self):
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        student = self.student
        xmldict = self.xmldict
        # Provider data
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        # Institution data
        xmldict['institution_acct'] = '000000000000'
        xmldict['institution_bank_id'] = '00'
        xmldict['institution_amt'] = '0.0'
        provider_amt = 0.0
        self.pay_item_id = '0000'
        if self.context.p_category == 'schoolfee':
            if not self.context.p_item == 'Balance':
                provider_amt = 1500.0
            if student.current_mode.endswith('_ft'):
                self.pay_item_id = '5700'
                if student.current_mode in ('ug_ft','de_ft','ume_ft'):
                    xmldict['institution_acct'] = '2017506430'
                    xmldict['institution_bank_id'] = '8'
                elif student.current_mode in ('dp_ft','dp_pt','ct_pt','ct_ft'):
                    xmldict['institution_acct'] = '0058653774'
                    xmldict['institution_bank_id'] = '72'
                elif student.current_mode in ('pg_ft'):
                    xmldict['institution_acct'] = '6230033428'
                    xmldict['institution_bank_id'] = '51'
            elif student.current_mode.endswith('_pt'):
                self.pay_item_id = '5701'
                if student.current_mode in ('ug_pt','de_pt'):
                    xmldict['institution_acct'] = '0122009929'
                    xmldict['institution_bank_id'] = '16'
                elif student.current_mode in ('pg_pt', 'special_pg_pt'):
                    xmldict['institution_acct'] = '4150033274'
                    xmldict['institution_bank_id'] = '51'
            elif student.faccode == 'JUPEB':
                self.pay_item_id = '5718'
                xmldict['institution_acct'] = '0231462068'
                xmldict['institution_bank_id'] = '16'
        elif self.context.p_category == 'clearance':
            self.pay_item_id = '5702'
            provider_amt = 1500.0
            if student.faccode == 'FCETA':
                xmldict['institution_acct'] = '5210007943'
                xmldict['institution_bank_id'] = '51'
            else:
                xmldict['institution_bank_id'] = '10'
                xmldict['institution_acct'] = '0031716047'
        elif self.context.p_category == 'gown':
            self.pay_item_id = '5704'
            xmldict['institution_bank_id'] = '16'
            xmldict['institution_acct'] = '0122011401'
        elif self.context.p_category.startswith('bed_allocation'):
            self.pay_item_id = '5716'
            students_utils = getUtility(IStudentsUtils)
            stage = students_utils.getAccommodationDetails(student)['bt']
            stage = stage.split('_')[2]
            if stage == 'fr':
                union_dues = 500.0
            else:
                union_dues = 300.0
            xmldict['institution_bank_id'] = '129'
            xmldict['institution_acct'] = '0014419432'
            xmldict['union_dues'] = 100 * union_dues
        elif self.context.p_category.startswith('hostel_maintenance'):
            self.pay_item_id = '5705'
            xmldict['institution_bank_id'] = '129'
            xmldict['institution_acct'] = '0014419432'

        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['institution_item_name'] = self.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmldict['institution_amt'] = 100 * (
            self.context.amount_auth - provider_amt - GATEWAY_AMT)

        if self.context.p_category == 'bed_allocation':
            xmldict['institution_amt'] = 100 * (
                self.context.amount_auth - GATEWAY_AMT - union_dues)
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="UNION DUES" item_amt="%(union_dues)d" bank_id="31" acct_num="0005986952" />
</item_details>
</payment_item_detail>""" % xmldict
        elif provider_amt == 0:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        elif self.context.p_category == 'schoolfee' and \
            student.current_mode == 'ug_ft':
            xmldict['institution_amt'] = 100 * (
                self.context.amount_auth - provider_amt - GATEWAY_AMT - 3000)
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
<item_detail item_id="3" item_name="T-SHIP" item_amt="200000" bank_id="72" acct_num="0061647254" />
<item_detail item_id="4" item_name="MTN-NET LIBRARY" item_amt="100000" bank_id="8" acct_num="20177722298" />
</item_details>
</payment_item_detail>""" % xmldict
        else:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT
        return

class CustomInterswitchPageApplicant(InterswitchPageApplicant):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomApplicantOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    pay_item_id = '5703'
    product_id = PRODUCT_ID

    def update(self):
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        xmldict = {}
        provider_amt = 400.0
        if self.applicant.applicant_id[:3] in ('pga', 'pgd'): # CERHI
            xmldict['institution_acct'] = '1014312532'
            xmldict['institution_bank_id'] = '117'
        elif self.applicant.applicant_id.startswith('pg'):
            xmldict['institution_acct'] = '0031716030'
            xmldict['institution_bank_id'] = '10'
        elif self.applicant.applicant_id.startswith('dp'):
            xmldict['institution_acct'] = '9201805071'
            xmldict['institution_bank_id'] = '17'
        elif self.applicant.applicant_id.startswith('pt'):
            xmldict['institution_acct'] = '9201805071'
            xmldict['institution_bank_id'] = '17'
        elif self.applicant.applicant_id.startswith('pre'):
            xmldict['institution_acct'] = '1014315131'
            xmldict['institution_bank_id'] = '117'
        elif self.applicant.applicant_id.startswith('cbt'):
            xmldict['institution_acct'] = '5030058259'
            xmldict['institution_bank_id'] = '51'
            provider_amt = 200.0
        elif self.applicant.applicant_id == 'afak':
            xmldict['institution_acct'] = '1014446565'
            xmldict['institution_bank_id'] = '117'
        else:
            xmldict['institution_acct'] = '1014326805'
            xmldict['institution_bank_id'] = '117'
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        xmldict['institution_amt'] = 100 * (self.context.amount_auth - provider_amt - GATEWAY_AMT)
        xmldict['institution_item_name'] = self.context.p_category
        xmldict['institution_name'] = INSTITUTION_NAME
        # Interswitch amount is not part of the xml data
        xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT
        return

class CustomInterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """ Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL

class CustomInterswitchPaymentRequestWebservicePageApplicant(
    InterswitchPaymentRequestWebservicePageApplicant):
    """ Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL 
